package com.jakubu9333.grammarManager.dao;

import com.jakubu9333.grammarManager.config.XmlDataConfig;
import lombok.NoArgsConstructor;
import org.basex.api.client.ClientSession;
import org.basex.core.Command;
import org.basex.core.cmd.Add;
import org.basex.core.cmd.CreateBackup;
import org.basex.core.cmd.Delete;
import org.basex.core.cmd.XQuery;
import org.springframework.http.HttpStatus;
import org.springframework.web.server.ResponseStatusException;

import java.io.IOException;
import java.util.UUID;

/**
 * @author Jakub Uhlarik
 */
@NoArgsConstructor
public class XmlDataDao {

    private ClientSession session;


    public static String xQueryRuleByLeft(String id) {
        return "descendant::*:grammar/*:rule[(@id = \"" + id + "\")]";
    }

    private static String xQueryGrammarByRoot(String root) {
        return "descendant::*:grammar[(@root = \"" + root + "\")]";
    }


    public XmlDataDao(ClientSession session) {
        this.session = session;
    }

    private String executeCommand(Command command) {
        try {
            return session.execute(command);
        } catch (IOException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }

    public String getByUri(String uri) {
        return executeCommand(new XQuery("db:open('"+XmlDataConfig.DB_NAME+"','" + uri + "')"));
    }

    public String getByRoot(String root) {
        return executeCommand(new XQuery(xQueryGrammarByRoot(root)));
    }

    public String getRuleByLeft(String id) {
        return executeCommand(new XQuery(xQueryRuleByLeft(id)));
    }

    public String createBackup() {
        return executeCommand(new CreateBackup(XmlDataConfig.DB_NAME));
    }


    public String add(String xml) {
        UUID uuid = UUID.randomUUID();
        executeCommand(new Add(uuid + ".grammar.xml", xml));
        return uuid + ".grammar.xml";
    }

    public String allUris() {
        return executeCommand(new XQuery("array{db:list(\""+XmlDataConfig.DB_NAME+"\")}"));
    }

    public String delete(String uri) {
        return executeCommand(new Delete(uri));
    }

    public String getByJsgfName(String name){
        //ignore two grammars with same name
        return executeCommand(new XQuery("head(descendant::*:grammar/*:meta[@name='jsgfName'][@content='"+name+"']/..)"));
    }

    public String deleteAll() {
        return null;
    }
}
